$NOMOD51
$NOLIST
$INCLUDE(REG752.PDF)
$LIST
$TITLE(HP752.ASM- ADCM-48 LISTING)
;********************************************************************
; ADCM-48 SOFTWARE
;
; Data acquistion and control system for HP-48 series scientific
; calculators. System provides 4 analog inputs, 4 digital inputs,
; 4 digital outputs.  Hardware is 87C752 at 3.6864 Mhz.
;
; Author:        John Wettroth
; Start:         August 1990 (breadboard)
; Beta Release:  March 1991
; Com'l Release: Octover 1991
;
; Revision and testing history follows including all
; breadboard versions
;
;********************************************************************
;                      REVISION HISTORY
;********************************************************************
;********************************************************************
; revsion -1  -  August 15, 1990   prerelease breadboard test version
;                                  proof of concept- processor comparisons
;
; This is a debug version of code for a HP-48 peripheral for
; doing A/D conversion and digital I/O on an HP-48.  Present plans
; include:
;       2 autoranging 8 bit A/D's- range 0 to 50 volts
;       1 8 bit PWM/DAC output
;       8 digital outputs- open collector 500 ma
;       4 opto-isolated digital inputs, 2-30 volts in
;
; Commands will be sent from the HP-48 to the peripheral.  System
; will do something and send back data or ACK.  Commands are single
; byte long with two nybbles, high and low.  High defines type of command
; , low defines data for command.
;
;   Currently Command set is:
;       0x- Attention- reset, wake up get ready (x is don't care)
;       1x- Read A/D x (x 0 to 4)
;       2x- Input dig in port (x is don't care)
;       3x- Output dig out port (x is data out from 0 to 15)
;
;   Any other upper nybbles force ignoring of command
;
; Two data logger modes are also being considered:
;
;       1. Send data logger command to unit with a time between
;          samples,T, unit will send ACK and then will send V1, V2
;          and digital in's back to HP-48 every T.  T could be as
;          often as 30 per second.  Send AT (attention) to abort
;
;       2.  An alternative mode would be similar to the above but
;           data is sent to an HP printer and T is probably limited
;           to every 5 seconds or so.  This wouldn't really cost
;           anything.  Send AT to abort.
;
; Have also considered making the system a Kermit server but this
; seems like a big hassle.  Good part is that variables would come
; back in proper format.  Think about it. (read)  Probably too much
; overhead, just do byte transfers.
;********************************************************************
;********************************************************************
;See HP48IR.asm and HPXTEST.ASM FOR MORE DESCRIPTION OF BB SOFTWARE
;********************************************************************
; revision 0- august 20, 1990 start   prerelease- post conceptual
;             august 28, 1990 finish
; jmw code and debug
;****
; System constants adjusted from HPRX.asm for 3.684 MHZ xtal.  This
; freq allows 60 hz interupt for sampling on usused timer.  This
; version  just debugged comm circuits, for debug purposes- system
; echoes characters out serial port with hp protocol and changes case
; THIS VERSION IS RENAMED TO HP752DEB.ASM FOR STORAGE
;********************************************************************
; revision 1- august 28, 1990 start    prerelease- refinement
; changed port pin mapping- xmit now on p1.6
; added tick timer int that comes in at 60hz using timer I
; this version ended with a version that reads ADC1 every .5 seconds
; and squirts to hp48.  All works basically-
; THIS VERSION RENAMED HP752AD2.ASM FOR STORAGE
;********************************************************************
; revision 2- september 6, 1990        prerelease- refinement
; revised to include sending commands to unit and having it respond
; supported commands are 0, 1, 2 and 3.
;********************************************************************
; revision 3- september 29, 1990       prerelease- refinement
; changed equates for final hardware configuration - sleep added "F"
;********************************************************************
; revision 4- november 15, 1990        prerelease- producibility
; changed equates and i/o routines for truly final hardware on pcb
;  big changes
;    port 3.0 is now irxmit- high to xmit
;    port 0.4 is now lsb of output- also a pwm to be done later
;    port 1.6 is now lsb of input- also an int to be done later
;    port 1.7 is now poff
;    port 3.4 is now a spare
;    port 0.3 is now a pilot light- flashes on comm
;********************************************************************
; revision 5- january 4, 1991         prerelease- refinement
;  this version will fix a bug, add a new pwm command, implement auto
;  shut-off and do something for low battery (undetermined).
;
;  notes:
;    new intersil op-amp allows solid operation down to 5.9V! at very
;  low voltages, system will interpret normal commands as sleep
;  commands and system will sleep often- may look into this further?
;    timer mode does not appear possible, there is no counter available
;  and i can't think of any reliable way to make sure that fast rates
;  on interupt line will not cause system to miss comm. at least if I
;  make sure, then system will miss counts during comm, un sat- will
;  not be included.
;    pwm mode will be included. kind of screws up neat nybble based
;  command set in that pwm is 8 bits. will create a "dblbyt" flag that
;  will be used for sending this data. (oh well, it was clean)
;
;  bug fix- missing recieve commands
;   there appears to be a bug relating to missing chars. the hypothesis
;   is that an interupt comes for a/d scanning when a character is
;   being recieved. the probability looks like about 1 in 20. solution
;   is to disable all other interupts when recieving. this will throw
;   off time slightly- but not enough to worry. character receive int
;   is only critical timing interupt.
; this version never completed- major flaw found- stored as hp752_5.asm
;********************************************************************
; revision 6- january 5, 1991           prerelease- refinement
;  a major revelation in fixing bug of version 5 is interupt latency
;  @3.6 megahertz adds up over a char to more than a half a bit time.
;  this version will change comm receive so that system is busy banging
;  bits from time int goes low to end of char (4.2 ms), this frees up
;  up counter which is the good news. more later.
;
;  counter changed equates slightly and added white wires-
;   reverse p1.6 and p1.7 (pins 21 and 22)
;********************************************************************
; revision 7- march 2, 1991-             *** beta release ***
; formal test version code with bug fixes
; 1. Module Test- all ok except digital out as noted below
; -fixed a gnarly bug in digital output. was writing same value back
; to dig ins as was on them in a subtle way. system would then draw
; gobs of power when digital inputs went high. effect would only
; show up when ins and outs used in same session
;
;********************************************************************
; revision 8- june 16, 1991-             beta release- refinement
; bug fix and feature enhancements
; Bug Fix-
; fixed a bug where approx 4 out of 1000 messages could get missed.
; apparently caused by timer int hitting at beginning of ir int. put
; a global disable as first instruction of ir int. no errors in approx
; 10000.  also moved extra check for r48b at beginning of a/d code.
; considering eliminating polling loop for r48b and just having int
; directly vector to decode logic. errors got very low so never done.
; also added "send all" a/d command when low nyb=5.(scotty thompson).
; also changed width of xmit pulses to 52 us nominal.  found that wider
; pulses incompatible with hp95. if old units are to get upgrades, this
; is the version. s/w will diverge from here. new versions
; will start at 10.  8 versions will proceed by tenths from here.
; also added support for 24 bit counter mode with code 5 of count read
;
; *** this version beta tested by Kenner and Thompson- unreleased
;********************************************************************
; revision 9- june 24, 1991-                  unreleased- refinement
; beta test comments of revision 8 code
;********************************************************************
; revision 10- june 24, 1991-                 unreleased- refinement
; new hardware fixes/new pcb revsion- final hardware
;
;   hardware changes:
;     state of irxmit is now active low
;     state of pilot is also active low
;     4 inputs get inverted on input
; never released- didn't want two hardware versions to diverge.
; created version 11 which includes macros for board revision.
; tested in house
;********************************************************************
; revision 11- JULY 28, 1991              beta 2- producibility
; - added macro set for board revision
; this will keep the software from diverging from beta units
; tweaked memory map slightly- old system used B reg for bits
; also added new command HPPRT- code 208+ that sends 4 a/d
; values to hp printer at 5 sec through 1 hour intervals
; fixed version 8 bugs with r0 usage and long jumps
; formal testing- no anomalies
;  released beta version 5 copies-
;  Kenner, Thompson, Meister, SAIC, Hyman
;********************************************************************
; revision 12- October 11, 1991-          commercial release
; cleanup of version 11 beta comments primarily:
; added code to isr to keep hp printer mode accurate- now 5.002 sec
; removed leading zero suppression from printf- reads better
; changed format of hp printing- added spaces for readability
; lengthened auto power off time to 10 minutes- agrees with manual
;  formal testing- no anomalies found
;********************************************************************
; revision 14- October 11, 1994-           limited release (2 copies)
; added code for time interval measurement mode- cust request
; customer will beta test for possible inclusion in future version
;
; testing- module and interactions- no anomalies found
;********************************************************************
; revision 15- November 7, 1994-	   commercial release
; special mode bit added- user feedback
; added code that requires that sleep be disabled before hp printer
; data logger or timer mode can be used.  one customer using wake from
; sleep code reported that he was able on rare ocassian to get system
; to inadventently enter hp printer mode on first power up due to a
; comm error.
; testing- module and interactions- no anomalies found
;********************************************************************
;
; MACRO DEFINITION USED FOR HARDWARE DEFINITION- ver 11
;  global macro for defining hardware revision of pwb.  proto board
;  digital outputs were inverted for IR LED and Pilot Led.  A/D mapping
;  was changed in rev1 board to improve noise pickup/routing.
;
; macro used to keep hardware versions from diverging and to maintain
; compatibility with original hardware beta test units for test
; consistency.
;
; BRDREV=0 IS PROTOTYPE BOARD WITH NO SILK SCREEN (QTY 25)
; BRDREV=1 IS PRODUCTION BOARD
;********************************************************************
%SET(BRDREV,1)
;********************************************************************
;
;
; EQUATES AND DATA DECLARATIONS
;
;Memory and bit locations
;
; bits- byte address 20H- do not use 20/21H
R48B    BIT     0H             ;bit 0- byte recieved
X48B    BIT     1H             ;bit 1- xmtr busy
JIFF    BIT     2H             ;bit 2- bit for a/d sample
TUP     BIT     3H             ;bit 3- time up bit- autopower off
DBLBYT  BIT     4H             ;bit 4- dbl byte bit for long commands
NSLP    BIT     5H             ;bit 5- disable sleep bit
SBIT    BIT     6H             ;bit 6- save bit for HP printer stuff
ZSB     BIT     7H             ;bit 7- zero suppr. bit for printf
;address 21 bits
PFLG    BIT     8H             ;bit 8- hp printer active flag
PTUP    BIT     9H             ;bit 9- hp printer time to print
			       ;       (v14- timer uses as well)
; more address 21 avail for bits
;
;** second bank of registers is available- 8 through F
;reminder never use Bank 0 R0- used by a/d isr
;
;A/D buffer locations- these are actually remapped by lookup later
;when values are requested-
ABUF    EQU     10H            ;ad0 beginning of adc buffer- len 5
;               11h            ;ad1
;               12h            ;ad2
;               13h            ;ad3
;               14h            ;ad4- (battery)
;
;Interger Math and HP Printer Storage
CHR     EQU     15H            ;char to send
PAR     EQU     16H            ;parity build register
BEGREG  EQU     17H            ;beginning of math storage
RC16    EQU     BEGREG+1       ;rc16 register- points to lsb
RD16    EQU     BEGREG+3       ;rd16 register- points to lsb

; timer mode variables- version 14
TMHUN   EQU     1BH            ;timer hundredths  10ms  each to 99
TMSEC   EQU     1CH            ;timer seconds     1s    each to 99
TMHSC   EQU     1DH            ;timer hundreds    100s  each to 255
; rem 1D/1E/1F avail

; DATA STORAGE LOCATION 22-2FH (20/21 USED AS BIT ADDRESSABLE)
COMERR  EQU     22H            ;received comm error ct
RBUF    EQU     23H            ;received byte buffer- length 1
XCTR    EQU     24H            ;counter for xmit bits
TICK    EQU     25H            ;tick counter low
RDLY    EQU     26H            ;receive delay counter

PDST    EQU     27H            ;print delay value storage

TIMO    EQU     28H            ;timout counter- 255*2.55 scs
PMIN    EQU     28H            ;PRT mins counter- ALT
S255    EQU     29H            ;timout- # of .01 scs
PSEC    EQU     29H            ;PRT secs counter

CHCLO   EQU     2AH            ;char count low
PCNTL   EQU     2AH            ;low bytes PRT alt def hpdl
CHCHI   EQU     2BH            ;char count high
PCNTH   EQU     2BH            ;hi bytes PRT alt def hpdl

TOFL    EQU     2CH            ;high high bytes of counter
RBTCTR  EQU     2DH            ;counter for rcv bits
DLY1    EQU     2EH            ;dlyctr, transient
DLY2    EQU     2FH            ;dlyct, transient, xmit and hp

;
;Constants- recieve bit banging
; fbit depends on board revision
; C added to recieve circuit- phototransistor was slower
; and had the same effect- test these values further
; fbit is the first bit delay value- accounts for latency and centering
; nbit is for subsequent bits- same for both boards

FBIT    EQU     %IF(%BRDREV EQ 0)THEN(3BH)ELSE(42H)FI
NBIT    EQU     3DH            ;delay for rest of bits

;Constants- transmit bit banging- these have changed in v8
;old versions too wide- found would not talk to hp95
DBITO   EQU     7D             ;on bit period (52 us)
DBITP   EQU     51D            ;delay between bits
;
;Constants for HP Printer IR bit banging
; All are special delays- no timers used
SHALF   EQU     25             ;start bit half bit delay
FDEL    EQU     17             ;first bit delay- special
PODEL   EQU     61             ;delay after a 1 bit sent
PRDEL   EQU     65             ;delay befor a 0 bit sent
DONDEL  EQU     17             ;delay after 1 or 0 bit sent
;
;Port Pin Assignments
PON     BIT     P1.6           ;power kill pin- low to kill
PMOUT   BIT     P0.4           ;pwm output
XMLT    BIT     P3.0           ;IR led on pin (brd 0 high,brd 1 low)
PILOT   BIT     P0.3           ;led pilot light (brd 0 high,brd 1 low)
;
;********************************************************************
;
;begin code-
        ORG     0H             ;absolute start
        AJMP    INIT           ;over constants, isr's
;
; ISR Stubs or Vectors
;
        ORG     03H            ; ext 0 vector- start bit
        AJMP    STRT           ; isr
;
        ORG     0BH            ; timer 0 stub
        RETI                   ;
;
        ORG     13H            ; ext 1 stub
        RETI
;
;this interupt has to be in place to minimize latency for comm
; will set jiff bit every 100th of a second for incrementing a/d
; a/d will then be fresh every 20th of a second
;
        ORG     1BH            ; timer i stub
        SETB    CLRTI          ;clr TI in flag- manual
        DJNZ    TICK,NOSC      ;dec the tick value
        MOV     TICK,#03D      ;reload tick time- .01 sec
        SETB    JIFF           ;.01 sec's = 0

        JNB     PFLG,NOSC      ;check hpprt flag

        DJNZ    DLY1,NOSC      ;dec 1/100 ths
        MOV     DLY1,#100d     ;reset 1/100 ths
        DJNZ    PSEC,NOSC      ;dec and test seconds
        MOV     PSEC,#60d      ;preload for jump
        DJNZ    PMIN,NOSC      ;dec and test minutes
        SETB    PTUP           ;set bit- time to print
NOSC:   RETI                   ;done- return for int
;

;Initialization code- does the following
;   clear state counter, R48R flag
;   setup intr for int0 last
;
INIT:   MOV     SP,#30H        ;initialize stk '752 only has 40h! 
        CLR     PMOUT          ;low on PMOUT
        MOV     P3,#0FH        ;output mosfet's off- p3 high input
        %IF(%BRDREV EQ 0)THEN(CLR     PILOT)ELSE(SETB    PILOT)FI
        %IF(%BRDREV EQ 0)THEN(CLR     XMLT)ELSE(SETB    XMLT)FI
        MOV     IE,#0H         ;make sure
        CLR     EA             ;'make sure'                       
        MOV     A,ADCON        ;get non bit adcon                 
        ORL     A,#20H         ;set adcen bit                     
        MOV     ADCON,A        ;put it back                       
        MOV     A,ADAT         ;dummy read to start               
        MOV     R0,#ABUF       ;setup a/d buffer pointer r0       
        MOV     TCON,#4CH      ;counter 0 and edge on int 0
        MOV     TH0,#0         ;clear counter                     
        MOV     TL0,#0         ;clear low                         
        MOV     TOFL,#0        ;clear counter overflow
        MOV     PWENA,#0       ;clr pwm
        SETB    TIRUN          ;300 hz interupt on timer1         
;INITIALIZE RAM VARIABLES
        MOV     TICK,#03D      ;setup tick timer- 1/100 second
        MOV     20H,#0H        ;FLAGS- make sure bits are clear
        MOV     21H,#0H        ;FLAGS- make sure bits are clear
        MOV     COMERR,#0      ;received comm error ct
        MOV     CHCLO,#0       ;received char cnt lo
        MOV     CHCHI,#0       ;received char cnt hi
        MOV     TIMO,#0EAH     ;timout counter- 10 mins
        MOV     S255,#0        ;timout low- # of .01 scs- to 2.55
;all initializations done- enable interrupts
        SETB    EX0            ;enable comm ints   
        SETB    ETI            ;enable jiff counter
        CLR     IE0            ;int0 cleared init
        SETB    CLRTI          ;clr TI in flag
        SETB    EA
;begin regular processing
; wait here for either a timer interrupt- fall into regular code
;
WAIT:   JB      R48B,DECD      ;handle command if buffer full
        JB      JIFF,AD1       ;if .01 sec elapsed then do next a/d
        JNB     TF0,WAIT       ;check overflow bit
        CLR     TF0            ;clear overflow bit
        INC     TOFL           ;inc high high bit
        AJMP    WAIT           ;jmp to wait for next
;end of main code loop- check for sleep and dispatch
AD1:    JB      R48B,DECD      ;precedence for comm buffer
        CLR     JIFF           ;clr jiff flag
        JB      NSLP,NTIM      ;dec poff vars if not disabled
        DJNZ    S255,NTIM      ;dec 2.5 sec ctr
        DJNZ    TIMO,NTIM      ;dec poff timer
        AJMP    OFF            ;then sleep
NTIM:   ACALL   ISAD           ;call increment and start a/d
        CLR     JIFF           ;clear jiff- done
        AJMP    WAIT           ;jmp to wait for next

;
;***************************************************************
;ISAD routine- keeps a buffer at abuf (10h) with fresh a/d values,
;              when value is requested, table lookup is value sent
;              stash "last" a/d value into buffer
;              increment a/d to next address and start
;
ISAD:   MOV     @R0,ADAT       ;stash last a/d result to buffer
        INC     R0             ;increment and check a/d pointer
        CJNE    R0,#ABUF+5,N5I ;check for adc#5
        MOV     R0,#ABUF       ;restart adc buffer pointer
N5I:    MOV     A,R0           ;get address of r0
        ANL     A,#07H         ;get 3 low bits                      
        MOV     R4,A           ;temp stash a
        MOV     A,ADCON        ;get non addressable adcon 
        ANL     A,#0F8H        ;kill low bits
        ORL     A,R4           ;put bits on a
        MOV     ADCON,A        ;get sfr for a/d start
        ORL     A,#8H          ;have to set this after
        MOV     ADCON,A        ;put out on adcon
        RET
;end of isad subroutine
;********************************************************************
;
;DECD routine- decodes high nybbles of instructions and squirts
; back appropriate response- arrive here by having a full buffer
;
; commands- nybbles
;       high    low
; --------------------------------------------------------------
;       00      none           reset/wakeup adcm
;       01      chan           read a/d n 0 thru 4-
;                              code- 5 send all (>v8*)
;       02      none           read inputs
;       03      data           output a nybble- with pwm support
;       04      code           counter
;                              code- 0 send low/high
;                                    1 stop counter
;                                    2 start counter
;                                    4 reset
;                                    5 read high high byte(>v8*)
;       05      code           pwm- 'multiple byte message'
;                              next byte sent is pwm value
;                              code  1 disable (single byte)
;       06      none           freq counter- returns 2 in 1 sec
;       07      chan           burst a/d1- send 131 values in 1.31 s
;       08      chan           burst a/d2- send 131 after input 1 falls
;       09                     unused- error
;       0A                     unused- error
;       0B                     unused- error
;       0C      code           timer mode (v14)
; ** note requires that sleep be disabled first
;                              code-0 arm timer- start rise/end fall dig 1
;                                   1 abort timer (send finish)
;                                   2 read timer- send low/high
;                                   3 read ticks
;
;       0D      code           hp printer data logger(>v11)
; ** note requires that sleep be disabled first
;                              code-0 every  5 seconds
;                                   1 every 15 seconds
;                                   2 every 30 seconds
;                                   3 every 60 seconds
;                                   4 every  5 minutes
;                                   5 every 15 minutes
;                                   6 every 30 minutes
;                                   7 every 60 minutes
;
;       0E      code           diagnostics
;                              code-0 send firmware version
;                                   1 send msg & comm err cnts
;       0F      code           sleep
;                              code-0 sleep now
;                                   1 disable sleep
;				    2 enable special commands
;**********************************************************
DECD:   MOV     A,RBUF         ;pick up command
        MOV     R3,RBUF        ;temp storage for rbuf
        CLR     R48B           ;clear the bit
        JNB     DBLBYT,SNGL    ;check dbl byte bit- pwm
        CLR     DBLBYT         ;clear bit
        AJMP    PWMVAL         ;jmp to pwm value
SNGL:   SWAP    A              ;get high nybble of a
        ANL     A,#0FH         ;mask it off
        RL      A              ;ready for jump table *2
        MOV     DPTR,#DTBL     ;load dptr with jmp table base
        JMP     @A+DPTR        ;jmp into jmp table

DTBL:   AJMP    RCMD           ;jmp to cmd code 0
        AJMP    ADCMD          ;jmp to cmd code 1
        AJMP    INCMD          ;jmp to cmd code 2
        AJMP    OCMD           ;jmp to cmd code 3
        AJMP    CNTR           ;jmp to cmd code 4
        AJMP    PWM1           ;jmp to cmd code 5
        AJMP    FREQ           ;jmp to cmd code 6
        AJMP    BRST1          ;jmp to cmd code 7
        AJMP    BRST2          ;jmp to cmd code 8
        AJMP    BCMD           ;jmp to cmd code 9- error
        AJMP    BCMD           ;jmp to cmd code A- error
        AJMP    BCMD           ;jmp to cmd code B- error
        AJMP    TCMD           ;jmp to cmd code C- timer (v14)
        AJMP    HPDL           ;jmp to cmd code D
        AJMP    DIAG           ;jmp to cmd code E
        AJMP    SLEEP          ;jmp to cmd code F

BCMD:   INC     COMERR         ;if none then must be err
        AJMP    WAIT           ;none of the above go back
;
; done with jump table
; **************************************************************
; COMMANDS
; **************************************************************
; ***** code 0
;RESET COMMAND- clears immediate and jumps to init code
RCMD:   AJMP    INIT           ;jump to initialization code
;end of reset command *****
;
; ***** code 1
;A/D COMMAND- A/D INPUTS ARE SCRAMBLED FROM CONNECTOR TO
;             INPUTS, ADDED CODE TO SEND ALL

ADCMD:  MOV     A,R3           ;get cmd byte again       
        ANL     A,#07H         ;mask off low bits + 1
        CJNE    A,#05,NRMAD    ;see if user wants all a/d val's
        CLR     A              ;send all a/d values
        MOV     R3,A           ;temp store a
S5LP:   MOV     A,R3
        MOV     DPTR,#SCTBL    ;load table base address
        MOVC    A,@A+DPTR      ;get address of selected
        MOV     R1,A           ;setup indirect R1
        MOV     A,@R1          ;pick up byte at offset
        ACALL   XMT48          ;send value
        INC     R3             ;inc ptr to next value
        CJNE    R3,#5,S5LP     ;done yet?
        AJMP    EXAD           ;done with send 5
NRMAD:  JC      OKAD           ;make sure valid
        INC     COMERR         ;bad value
        AJMP    EXAD           ;bail
OKAD:   MOV     DPTR,#SCTBL    ;load table base address
        MOVC    A,@A+DPTR      ;get address of selected
        MOV     R1,A           ;setup indirect R1
        MOV     A,@R1          ;pick up byte at offset
        AJMP    SEAD           ;send a/d value
;old board has different mapping than new board
%IF(%BRDREV EQ 0) THEN
        (
SCTBL:  DB      ABUF+2
        DB      ABUF+0
        DB      ABUF+3
        DB      ABUF+1
        DB      ABUF+4
        )
ELSE(
SCTBL:  DB      ABUF+3
        DB      ABUF+1
        DB      ABUF+2
        DB      ABUF+0
        DB      ABUF+4
        )FI                    ;board rev macro call

SEAD:   ACALL   XMT48          ;send byte to hp48        
EXAD:   AJMP    WAIT           ;bail- also error exit
;end of A/D command *****
;
; ***** code 2
;INPUT COMMAND- LOW BITS OF PORT 3 AND P1.7
; NEW BOARD HAS THESE BITS IN REVERSE ORDER ON P3
INCMD:
%IF(%BRDREV EQ 0)THEN
        (
        MOV     A,P3           ;get port 3
        MOV     C,P1.7         ;get lsb bit from p1.7- timer too
        ANL     A,#0EH         ;low nybble is input- lsb other
        MOV     ACC.0,C        ;put lsb into acc.0
        ) ELSE
        (
        MOV     B,P3           ;get port 3 to bit addr B reg
        MOV     C,P1.7         ;get lsb from p1.7
        MOV     B.0,C          ;build first in b
        MOV     A,B            ;put two into A
        MOV     C,B.3          ;get bit 3
        MOV     ACC.1,C        ;stash in A
        MOV     C,B.1          ;get bit 3
        MOV     ACC.3,C        ;stash in A
        ANL     A,#0FH         ;mask high bits
        XRL     A,#0FH         ;invert all bits
        )FI
        ACALL   XMT48          ;send byte to hp 48              
        AJMP    WAIT           ;bail                            
;end of input command *****
;
;
;
; ***** code 3
;OUTPUT COMMAND- HANDLES LOW BIT- with pwm
OCMD:   MOV     A,R3           ;restore received byte        
        ANL     A,#0FH         ;get low bits                 
        MOV     C,ACC.0        ;put lowest into carry for PMOUT
        CLR     ACC.0          ;get rid of low bit
        SWAP    A              ;put in high nybble           
        MOV     R3,A           ;stash a again                
        MOV     A,P3           ;get port 3                   
        ANL     A,#0FH         ;clear high bits              
        ORL     A,R3           ;put bits on it
        ORL     A,#0EH         ;make sure low bits set- bug fix
        MOV     P3,A           ;put back on port             
        MOV     PMOUT,C        ;put lsb out- tiny time lag...
ODONE:  AJMP    WAIT           ;bail
;end of output command *****
;
; ***** code 4
; COUNTER COMMAND- now supports 24 bit mode
CNTR:   MOV     A,R3           ;restore byte          
        ANL     A,#0FH         ;get low nybble        
        JNZ     CN0            ;check for zero- send low
        MOV     A,TH0          ;get high
        ACALL   XMT48          ;send high
        MOV     A,TL0          ;get low
        ACALL   XMT48          ;send low
        AJMP    CNEX           ;send and exit
CN0:    CJNE    A,#1H,CN1      ;check for 1- stop
        CLR     TR0            ;clear run bit         
        AJMP    CNEX           ;exit no send          
CN1:    CJNE    A,#2H,CN2      ;check for 2- start
        SETB    TR0            ;set run bit           
        AJMP    CNEX           ;exit no send          
CN2:    CJNE    A,#4H,CN4      ;check for 4- reset
        MOV     TH0,#0         ;reset high            
        MOV     TL0,#0         ;reset low             
        MOV     TOFL,#0        ;reset overflow byte
        CLR     TF0            ;reset overflow bit
        AJMP    CNEX           ;exit- no send
CN4:    CJNE    A,#5H,CN5      ;check for 4- reset
        MOV     A,TOFL         ;get high high byte
        ACALL   XMT48          ;send it
        AJMP    CNEX           ;done sending high byte
CN5:    INC     COMERR         ;bump error
CNEX:   AJMP    WAIT           ;exit                  
; end of counter command *****
;
; ***** code 5
; PWM COMMAND- MULTIPLE BYTE COMMAND (ONLY ONE)
PWM1:   MOV     A,R3           ;get byte recvd      
        ANL     A,#0FH         ;get low nybl        
        JZ      ENPW           ;if 1 then disable   
        MOV     PWENA,#0       ;disable             
        CLR     PMOUT          ;clr PMOUT
        AJMP    PWMEX          ;no second byte      
ENPW:   SETB    DBLBYT         ;set byte 2 bit      
        MOV     PWMP,#7        ;set prescaler 1khz  
        AJMP    PWMEX          ;exit                
PWMVAL: MOV     R3,A           ;get second byte     
        CPL     A              ;more natural        
        MOV     PWMCM,A        ;put in compare reg  
        MOV     PWENA,#1       ;enable              
PWMEX:  AJMP    WAIT                                
; end of pwm command *****
;
; ***** code 6
; FREQUENCY COUNTER COMMAND- STRANGE ONE
FREQ:   CLR     TIRUN          ;clr run bit for ti- .01 sec
        CLR     TR0            ;clr run bit for t0                
        MOV     TH0,#0         ;reset high                        
        MOV     TL0,#0         ;reset low
        MOV     TOFL,#0        ;reset high high
        CLR     TF0            ;reset overflow bit
        %IF(%BRDREV EQ 0)THEN(SETB    PILOT)ELSE(CLR     PILOT)FI
        CLR     EX0            ;disable comm ints for now
        CLR     JIFF           ;clr jiff flag                     
        SETB    TIRUN          ;run i counter again               
PREF:   JNB     JIFF,PREF      ;get i counter known
        CLR     TIRUN          ;stop                              
        CLR     JIFF           ;known state                       
        MOV     DLY1,#100D     ;100 jiff= 1 sec
        SETB    TR0            ;run counter 0                     
        SETB    TIRUN          ;go                                
FRQLP:  JNB     TF0,NOOVF      ;check overflow bit
        CLR     TF0            ;clear overflow
        INC     TOFL           ;inc high high bit
NOOVF:  JNB     JIFF,FRQLP     ;loop for 1/100 sec
        CLR     JIFF           ;clr jiff                          
        DJNZ    DLY1,FRQLP     ;loop for 1 sec
        CLR     TR0            ;stop counter                      
        MOV     A,TH0          ;get high
        ACALL   XMT48          ;send high
        MOV     A,TL0          ;get low
        ACALL   XMT48          ;send low
        %IF(%BRDREV EQ 0)THEN(CLR     PILOT)ELSE(SETB    PILOT)FI
        SETB    EX0            ;re-enable comm
        AJMP    WAIT           ;done                              
; end of frequecny command *****
;
; ***** code 7
; BURST 1 A/D COMMAND- SEND 131 A/D VALUES- ADDED TABLE LOOKUP
BRST1:  MOV     A,R3           ;get cmd byte again
        ANL     A,#07H         ;mask high bits + 1
        MOV     DPTR,#SCTBL    ;load table base address
        MOVC    A,@A+DPTR      ;get address of selected
        CLR     C              ;get ready to subtact
        SUBB    A,#ABUF        ;just want offset value
        MOV     R2,A           ;put in r2 for remainder
        %IF(%BRDREV EQ 0)THEN(SETB    PILOT)ELSE(CLR     PILOT)FI
        MOV     DLY1,#131D     ;setup byte counter
        MOV     A,ADAT         ;dummy read- clear
        MOV     A,ADCON        ;put it out to sfr
        ANL     A,#0F8H        ;clear low bits
        ORL     A,R2           ;reassert channel
        MOV     ADCON,A        ;reload a/d
        CLR     JIFF           ;start even
        MOV     TICK,#03H      ;start even on ticks
BSTLP:  MOV     A,ADCON        ;get sfr for start
        ORL     A,#8H          ;have to set this after
        MOV     ADCON,A        ;start a/d first time
BSDEL:  JNB     JIFF,BSDEL     ;wait 1/100 th sec
        MOV     A,ADAT         ;get a/d value
        ACALL   XMT48          ;send to hp48
        CLR     JIFF           ;ready jiff again
        DJNZ    DLY1,BSTLP     ;loop 100 times
        %IF(%BRDREV EQ 0)THEN(CLR     PILOT)ELSE(SETB    PILOT)FI
BSTEX:  AJMP    WAIT           ;go back to loop- DONE
; end of burst a/d #1
;
; ***** code 8
; BURST 2 A/D COMMAND- send 131 a/d values after input 1 falls
;                      waits for input to fall then jmps brst1
;                      cancel by any comm                     
BRST2:  %IF(%BRDREV EQ 0)THEN(JNB  P1.7,BR1J)ELSE(JB  P1.7,BR1J)FI
        JNB     R48B,BRST2     ;abort on comm            
        AJMP    BST2EX         ;exit on abort
BR1J:   AJMP    BRST1          ;jmp to regular brst
BST2EX: AJMP    WAIT           ;aborted exit on comm
; end of burst a/d #2

;***** code C
;timer mode
; require that sleep be disabled- send 0f1 (v15)
; after arm command sent (code 0)
; wait for digin to go high- start timer/light pilot
; wait for digin to go low- time out if > long time?
; when digin low- stop time/kill pilot/save time/send done
; time now stored-
; send tmsec/tmhun if code 1 sent
; abort any comm while timing/waiting
;
TCMD:	JB	NSLP,OKTCMD    ;check to see if sleep disabled
	AJMP	WAIT
OKTCMD: MOV	A,R3	       ;restore byte
        ANL     A,#01H         ;get low bit
        JNZ     TSND           ;check for zero- arm or send
;setup time out timeout for digin high- use printer logic
        SETB    NSLP           ;disable sleep- shared logic
        MOV     PMIN,#255      ;4 hours wait max
        MOV     PSEC,#0        ;0 seconds
        CLR     PTUP           ;clear print time flag
        SETB    PFLG           ;start process- 10 mins to ptup
; wait for dig in 1 to be high
TWTHLP: JB      PTUP,TABT      ;abort function if timeout
        JB      R48B,TABT      ;if any comm then abort
        %IF(%BRDREV EQ 0)THEN(JNB  P1.7,TWTHLP)ELSE(JB  P1.7,TWTHLP)FI

;get here if dig in high and no time out- light pilot
;now reset all times and wait for digin to fall
;reset abort timer and begin running
        %IF(%BRDREV EQ 0)THEN(SETB    PILOT)ELSE(CLR     PILOT)FI
        MOV     PMIN,#255      ;reload 255 minutes wait max/low
        MOV     PSEC,#0        ;0 seconds
        MOV     TMHUN,#0       ;zero hundredth timer
        MOV     TMSEC,#0       ;zero seconds timer
        MOV     TMHSC,#0       ;zero hundred seconds timer
        CLR     TIRUN          ;stop ticks for a jiff
        CLR     JIFF           ;start even
        MOV     TICK,#03H      ;start even on ticks
        SETB    TIRUN          ;start ticks again
;light pilot while timing
TMDEL:  JB      PTUP,TABT      ;abort function if timeout
        JB      R48B,TABT      ;if any comm then abort
        %IF(%BRDREV EQ 0)THEN(JNB  P1.7,TM1L)ELSE(JB  P1.7,TM1L)FI
        JNB     JIFF,TMDEL     ;wait 1/100 th sec
;increment time values- roll hundredth's to seconds
        CLR     JIFF           ;clear jiff flag
        INC     TMHUN          ;inc hundredths
        MOV     A,TMHUN        ;get copy
        CJNE    A,#100D,TMDEL  ;inc secs if hundredths =100
        MOV     TMHUN,#0       ;zero hundredths
        INC     TMSEC          ;inc seconds
        MOV     A,TMSEC        ;get copy
        CJNE    A,#100D,TMDEL  ;inc secs if hundredths =100
        MOV     TMSEC,#0       ;zero hundredths
        INC     TMHSC          ;inc hundreds- no wrap detect
        AJMP    TMDEL          ;jump back into loop

;code 1- send time values as stored
TSND:   MOV     A,TMHSC        ;get hundreds
        ACALL   XMT48          ;send low
        MOV     A,TMSEC        ;get seconds
        ACALL   XMT48          ;send high
        MOV     A,TMHUN        ;get hundredth's
        ACALL   XMT48          ;send low
        AJMP    TMEX           ;done- exit

;timer stopped normal- kill pilot then send byte
TM1L:   %IF(%BRDREV EQ 0)THEN(CLR     PILOT)ELSE(SETB    PILOT)FI
        MOV     A,#01H         ;finish normal- send byte to calc
        ACALL   XMT48          ;send to hp48
TABT:   CLR     PFLG           ;end timeout logic- abort entry
        %IF(%BRDREV EQ 0)THEN(CLR     PILOT)ELSE(SETB    PILOT)FI
TMEX:   AJMP    WAIT           ;go back to loop- DONE

;end of timer mode
;
;***** code D
; HP PRINTER DATA LOGGER
; require that sleep be disabled- send 0f1 (v15)
; sends data to hp 82240a/b printer every n seconds
; formatted as shown below
;
;        123456789012345678901234
; Header #_nnnn_*15s____Batt_x.y_(CR)
; codes:
;      0 every  5 seconds
;      1 every 15 seconds
;      2 every 30 seconds
;      3 every 60 seconds
;      4 every  5 minutes
;      5 every 15 minutes
;      6 every 30 minutes
;      7 every 60 minutes
;
HPDL:	JB	NSLP,OKHPDL    ;check to see if sleep disabled
	AJMP	WAIT
OKHPDL: CLR	EX0	       ;disable comm ints- turn off to exit
        MOV     A,R3           ;get cmd byte again
        ANL     A,#07H         ;mask high bits + 1
        MOV     PDST,A         ;stash value
        MOV     PCNTL,#0       ;zero low byte PRT alt def hpdl
        MOV     PCNTH,#0       ;zero hi byte PRT alt def hpdl
        ACALL   LDTM           ;load time values
        SETB    PFLG           ;start process
; time is now running and will automatically do a/d's
HPIDL:  JB      PTUP,TMPRT     ;hang waiting for time up
NOTM:   JNB     JIFF,HPIDL     ;not time- do a/d
        CLR     JIFF           ;clear .01 sec flag
        ACALL   ISAD           ;do convert
        AJMP    HPIDL          ;continue to hang
TMPRT:  ACALL   LDTM           ;reset timer
        CLR     PTUP           ;clear the ptime flag
;finally its time to print- yeah- light pilot
        %IF(%BRDREV EQ 0)THEN(SETB     PILOT)ELSE(CLR      PILOT)FI
        MOV     DPL,PCNTL      ;load dpl with low byte PRT's
        MOV     DPH,PCNTH      ;load dph with hi byte PRT's
        INC     DPTR           ;increment it
        MOV     PCNTL,DPL      ;stash dpl
        MOV     PCNTH,DPH      ;stash dph
        MOV     A,#'#'         ;load 'N- header
        ACALL   HPPRT          ;print it
        MOV     A,#' '         ;load space
        ACALL   HPPRT          ;print it
        MOV     RC16,PCNTL     ;put counts in low rc
        MOV     RC16-1,PCNTH   ;put counts in hi rc
        ACALL   TOBCD          ;convert to bcd
        MOV     B,#4           ;print 4 digits
        MOV     R7,#0          ;no digits after dp
        ACALL   PRINTF         ;squirt to printer
        MOV     B,#5           ;length of each entry
        MOV     A,PDST         ;get delay value
        MUL     AB             ;set a to offset
        MOV     B,A            ;stash value
        MOV     R7,#5          ;loop to send string
        MOV     DPTR,#UTBL     ;tim string table
ULP:    MOV     A,B            ;load a with offset
        MOVC    A,@A+DPTR      ;get value
        ACALL   HPPRT          ;print the value
        INC     B              ;inc offset
        DJNZ    R7,ULP         ;pound till 6 values
        MOV     DPTR,#BSTR     ;print batt string- 0 term
BTSLP:  CLR     A              ;clear acc for movc
        MOVC    A,@A+DPTR      ;get value
        JZ      BTSDN          ;end of string?
        ACALL   HPPRT          ;print string value
        INC     DPTR           ;inc pointer
        AJMP    BTSLP          ;continue looping
BTSDN:  MOV     A,ABUF+4       ;get battery volts
        ACALL   ML40           ;mul by .040 volts
        ACALL   TOBCD          ;convert to bcd
        MOV     B,#3           ;print 3 digits
        MOV     R7,#2          ;2 past the dp
        ACALL   PRINTF         ;print it
        MOV     A,#4           ;load a CR
        ACALL   HPPRT          ;print it
;header and carriage return now printed
        MOV     A,#' '         ;print space on new line
        ACALL   HPPRT          ;print it
        MOV     R3,#0          ;send values- prep
H4LP:   MOV     A,R3           ;setup ctr
        MOV     DPTR,#SCTBL    ;load table base address
        MOVC    A,@A+DPTR      ;get address of selected
        MOV     R1,A           ;setup indirect R1
        MOV     A,@R1          ;pick up byte at offset
        ACALL   ML20           ;mul value by .020
        ACALL   TOBCD          ;convert to bcd
        MOV     B,#3           ;print 3 digits
        MOV     R7,#2          ;two digits after dp
        ACALL   PRINTF         ;squirt to printer
        MOV     A,#' '         ;load a space
        ACALL   HPPRT          ;print it
        MOV     A,#' '         ;load a space
        ACALL   HPPRT          ;print it
        INC     R3             ;inc ptr to next value
        CJNE    R3,#4,H4LP     ;done yet?

        MOV     A,#04h         ;carriage return printer
        ACALL   HPPRT          ;print it
        MOV     A,#04h         ;carriage return printer
        ACALL   HPPRT          ;print it

        %IF(%BRDREV EQ 0)THEN(CLR     PILOT)ELSE(SETB    PILOT)FI
        AJMP     HPIDL         ;done with send 4- JMP IDLE
;units table
UTBL:   DB      ' * 5s'
        DB      ' *15s'
        DB      ' *30s'
        DB      ' *60s'
        DB      ' * 5m'
        DB      ' *15m'
        DB      ' *30m'
        DB      ' *60m'
;batt string
BSTR:   DB      '   Batt ',0h
; time sequence for sending lookup
TIMS:   DB      05d,15d,30d,60d
; *** LOCAL SUBROUTINE FOR LOADING TIME REGISTERS ***
LDTM:   CLR     EA             ;clr ints for now
        MOV     A,PDST         ;get value for loop
        ANL     A,#03H         ;look at two lowest bits
        MOV     DPTR,#TIMS     ;point to time table
        MOVC    A,@A+DPTR      ;get time value
        MOV     R4,A           ;temp stash a
        MOV     A,PDST         ;get value again
        JB      ACC.2,LMIN     ;mins or seconds?
        MOV     PSEC,R4        ;load value into seconds
        MOV     PMIN,#1        ;zero minutes- djnz preload
        AJMP    LDEND          ;done exit
LMIN:   MOV     PSEC,#1        ;load seconds- djnz prload
        MOV     PMIN,R4        ;load value into minutes
LDEND:  MOV     DLY1,#57D      ;first jiff ct- setup time adjust
        MOV     TICK,#2        ;tweak tick counter for last adj
        CLR     JIFF           ;start at init jiff-
        SETB    EA             ;re-enable ints
        RET                    ;return with values set
;***************************************************************
;
;***** code E
; DIAGNOSTICS COMMAND
DIAG:   MOV     A,R3           ;get cmd byte again             
        ANL     A,#0FH         ;mask high bits                 
        JNZ     CMSTAT         ;if non-zero then send errors   
        MOV     DPTR,#VSTR     ;get v string                   
VLP:    CLR     A              ;send string                    
        MOVC    A,@A+DPTR      ;get byte                       
        JZ      DDIAG          ;done?                          
        ACALL   XMT48          ;send it                        
        INC     DPTR           ;next letter                    
        AJMP    VLP            ;loop                           
;LOCAL STRING STORAGE
VSTR:	DB	169D,'TRAVTECH 94-V15',0H
CMSTAT: MOV     A,CHCHI        ;get char count high
        ACALL   XMT48          ;send it                  
        MOV     A,CHCLO        ;get com errors low
        ACALL   XMT48          ;send it                  
        MOV     A,COMERR       ;get com errors           
        ACALL   XMT48          ;send it                  
        MOV     COMERR,#0      ;clear error cnt          
        MOV     CHCLO,#0       ;clear char lo cnt        
        MOV     CHCHI,#0       ;clear char hi cnt        
DDIAG:  AJMP    WAIT           ;done                     
; end of diagnostics command *****                       
;                                                        
; ***** code F                                           
;SLEEP COMMAND                                           
SLEEP:  MOV     A,R3           ;get byte                 
        ANL     A,#0FH         ;get low nyb              
        JZ      OFF            ;if low then sleep        
	CJNE	A,#01,SKSLP    ;exactly one in low nybble
	SETB	NSLP	       ;disable sleep
			       ;also enable hpdl and timer (v15)
SKSLP:	AJMP	WAIT	       ;done
OFF:    CLR     PON            ;kill the power bit       
        AJMP    OFF            ;hang tight while we die  
;end of sleep command
;
;********************************************************************
;ISR CODE
;********************************************************************
;EXT INT0- FALLING EDGE OF START BRINGS YOU HERE
; THIS IS NEW RECEIVE ROUTINE- BANGS BITS TIL DONE
;********************************************************************
;
STRT:   CLR     EA             ;disable further ints
        PUSH    PSW
        PUSH    ACC
        JNB     P1.5,GDST      ;should be low still- jump
        AJMP    CMERR          ;bail out- bad start
GDST:   JNB     R48B,LPROC     ;last command processed?
        AJMP    CMERR          ;bail if not
LPROC:  %IF(%BRDREV EQ 0)THEN(SETB    PILOT)ELSE(CLR     PILOT)FI
        MOV     RBTCTR,#8      ;clear bit ctr- 8 after first
        MOV     RDLY,#FBIT     ;delay for a first bit time
RFBIT:  DJNZ    RDLY,RFBIT     ;do delay
RNXT:   CPL     P3.4           ;cpl debug
        MOV     C,P1.5         ;get receive bit
        RRC     A              ;put bit in place
        MOV     RDLY,#NBIT     ;delay for a normal bit time
RBIT:   DJNZ    RDLY,RBIT      ;do delay
        DJNZ    RBTCTR,RNXT    ;if 8 bits then done
        AJMP    CMDN           ;loop for next bit
CMERR:  INC     COMERR         ;inc comm error debug
        AJMP    CMEX           ;exit
CMDN:   MOV     RBUF,A         ;buffer byte to rbuf
        SETB    R48B           ;set byte recvd flag
CMEX:   MOV     TIMO,#0EAH     ;rst 10 min timer for pwr off
        MOV     DPL,CHCLO      ;get char count low byte
        MOV     DPH,CHCHI      ;get char coutn hi byte
        INC     DPTR           ;increment count
        MOV     CHCLO,DPL      ;restash low
        MOV     CHCHI,DPH      ;restah high
        POP     ACC            ;restore acc
        POP     PSW            ;restore psw
        MOV     RDLY,#20H      ;inter char delay
ICDEL:  DJNZ    RDLY,ICDEL     ;wait about 200 us
        CLR     IE0            ;clr int flag0- immediate int
        SETB    EA             ;re-enable ints
        %IF(%BRDREV EQ 0)THEN(CLR     PILOT)ELSE(SETB    PILOT)FI
        RETI
;
;end of ext interupt 0- receive processing
;
;********************************************************************
;SUBROUTINE XMT48- SEND A BYTE IN A USING HP CALC TO CALC PROTOCOL
;********************************************************************
;
XMT48:  CLR     EA             ;don't allow ints for now   
        PUSH    PSW            ;store status
        MOV     XCTR,#8        ;bit ctr                    
        %IF(%BRDREV EQ 0)THEN(SETB  XMLT)ELSE(CLR XMLT)FI
        ACALL   BITDEL         ;call delay                 
X48LP:  RRC     A              ;carry=lsb                  
        %IF(%BRDREV EQ 0)THEN(CPL     C)FI
        MOV     XMLT,C         ;put on port pin- hp inverted
        ACALL   BITDEL         ;call delay
        DJNZ    XCTR,X48LP     ;8 bits sent yet?           
        POP     PSW            ;restore status             
        CLR     X48B           ;transmit mt flag           
        MOV     DLY2,#50H      ;stop bit delay
XSTDL:  DJNZ    DLY2,XSTDL     ;hang for 500 uS
        SETB    EA             ;ints ok again              
        RET                    ;done                       
                                                           
; LOCAL BIT DELAY SUBROUTINE
;       bitdel- delay for the period that the led is on, kill led
;       then delay remainder of time- uses two equates            
;
BITDEL: MOV     DLY2,#DBITO    ;load light on del
BONDEL: DJNZ    DLY2,BONDEL    ;delay for bondel
%IF(%BRDREV EQ 0)THEN(CLR     XMLT)ELSE(SETB    XMLT)FI
        MOV     DLY2,#DBITP    ;load remain delay
BPDEL:  DJNZ    DLY2,BPDEL     ;delay it
        RET

;********************************************************************
;SUBROUTINE HPPRT- SEND A BYTE IN A USING HP PRINTER PROTOCOL
;********************************************************************
;HPPRT SUBROUTINE
;SENDS BYTE IN CHR TO HP PRINTER
HPPRT:  CLR     EA             ; disable int- critical timing
        MOV     PAR,#0H        ; parity storage
        MOV     CHR,A          ; character storage
        MOV     R4,#4H         ; counter                              
ECLP:   MOV     A,#0CH         ; mask table start offset
        ADD     A,R4           ; go into table by r4                  
        MOVC    A,@A+PC        ;                                      
        ANL     A,CHR          ; mask bits with byte to xmit          
        MOV     C,PSW.0        ; get parity                           
        MOV     A,PAR          ;                                      
        RRC     A              ; rotate into parity storage           
        MOV     PAR,A          ; stash                                
        DJNZ    R4,ECLP        ; do 4 times                           
        AJMP    SETUP
MSKTBL: DB      078H           ; mask for hp protocol error code      
        DB      0E6H           ; generation                           
        DB      0D5H           ;                                      
        DB      08BH           ;                                      
SETUP:  MOV     R6,#12         ; number of bits to send               
        ACALL   BURST          ; now ready to start
        MOV     R5,#SHALF
        ACALL   DELAY          ; start hbit delay
        ACALL   BURST          ;
        MOV     R5,#SHALF
        ACALL   DELAY          ; start hbit delay
        ACALL   BURST          ;
        MOV     R5,#FDEL       ; special first delay
        ACALL   DELAY          ; start hbit delay
;start sequence sent- send loop                                       
SNDLP:  CJNE    R6,#9,WBT      ; corr bits or normal?                 
WBT:    JC      NRM                                                   
        MOV     A,PAR          ; get parity                           
        RLC     A              ; get bit                              
        MOV     PAR,A          ; stash back                           
        MOV     SBIT,C         ; put in sbit                          
        AJMP    DOIT           ; go send it
NRM:    MOV     A,CHR          ; get character                        
        RLC     A              ; get bit                              
        MOV     CHR,A          ; stash back                           
        MOV     SBIT,C         ; put in sbit                          
        AJMP    DOIT           ; equalize time
;now have bit high or low to send                                     
DOIT:   MOV     C,SBIT         ; get bit                              
        JNC     NOBRST         ; one?                                 
        ACALL   BURST          ; send burst
        MOV     R5,#PODEL      ; post delay
        ACALL   DELAY          ;
        AJMP    DONE           ; done
NOBRST: MOV     R5,#PRDEL      ; pre delay
        ACALL   DELAY          ;
        ACALL   BURST          ;
DONE:   MOV     R5,#DONDEL     ; done delay
        ACALL   DELAY          ;
        DJNZ    R6,SNDLP       ; continue                             
        SETB    EA             ; ints ok
        RET                    ; really done                          

;LOCAL BURST SUBROUTINE
;generates a burst of 7 cycles of 30.7 khz- close as can be had       
BURST:  %IF(%BRDREV EQ 0)THEN(CLR     XMLT)ELSE(SETB    XMLT)FI
        MOV     R4,#0EH                                               
BLP:    CPL     XMLT
        NOP                                                           
        NOP                                                           
        DJNZ    R4,BLP                                                
        RET                                                           
                                                                      
;LOCAL DELAY SUBROUTINE
;can enter either delay or delay1- tweaking                           
DELAY:  NOP                                                           
DELAY1: DJNZ    R5,DELAY1
        RET                                                           
;
;**********************************************************
;MATH SUBROUTINES- MODIFIED FROM TOBCD AND PRINTF
;                   OF INTMATH (SHORTENED TO 16 BITS)
;**********************************************************
;
;**********************************************************
;ML20 subroutine- multiplies value in acc by ".020"
;                 value left in RC16 (actually 2)
;                 analog inputs to volts
ML20:   MOV     RC16-1,#0      ;clear high byte of RC16
        MOV     RC16,A         ;acc to low byte of RC16
;subroutine entry point- used by ml40
SUB20:  CLR     C              ;kill carry for rotate
        MOV     A,RC16         ;get low byte of RC16
        RLC     A              ;rotate bit
        MOV     RC16,A         ;put low byte in place
        MOV     A,RC16-1       ;update high byte
        RLC     A              ;rotate into a
        MOV     RC16-1,A       ;update high byte
        RET                    ;done
;
;**********************************************************
;ML40 subroutine- multiplies value in acc by ".040"
;                 value left in RC16 (actually 4)
;                 used for battery voltage
ML40:   ACALL   ML20           ;load a and mul by 2
        ACALL   SUB20          ;mul rc16 by 2 (4 total)
        RET
;
;**********************************************************
; TOBCD routine- converts bin in RC16 to BCD format and
;                leaves value in RD16. Supports numbers to 10k
;                technique is from INTEL 8048 app notes- slick
;
TOBCD:  MOV     RD16,#0        ;zero low byte bcd accum
        MOV     RD16-1,#0      ;zero high byte bcd accum
        MOV     R5,#16d        ;bit counter
BIGLP:  MOV     R1,#RD16       ;R1 now point to BEG of bcd
;BIN=BIN*2 (RC)
        ACALL   SUB20          ;RC16=RC16*2
;BCD=BCD*2+CARRY                                        
        MOV     R4,#2          ;byte counter
BCDLP:  MOV     A,@R1
        ADDC    A,@R1
        DA      A
        MOV     @R1,A
        DEC     R1
        DJNZ    R4,BCDLP
;LOOP
        DJNZ    R5,BIGLP
        RET
;done with bcd routine
;
;**********************************************************
;
; PRINTF-Writes out b chars with decimal point at r7 at
;        current cursor location from bcd number in RD16
;        make r7=0 if no dp wanted, dp is left justified
;        deleted zero suppression from old code
;        examples b=1,r7=1 would display '.X'
;                 b=3,r7=2 would display 'X.YZ'
;                 uses general flag locally-ZSB DP
;                 uses R1 and acc and one stack byte
;        assumes function to output a char is putchar
;
PRINTF:  CLR       C               ;set up subb- offset for dp
         MOV       A,B             ;get count into b                    
         SUBB      A,R7            ;set up decimal pt ctr               
         MOV       R7,A            ;stick back in r7                    
         INC       R7              ;equalize dp ctr-left just           
         MOV       A,B             ;set up ptr /2                       
         RRC       A               ;div 2                               
         JB        B.0,EVEN        ;must dec a if even ct               
         DEC       A               ;dec it else not                     
EVEN:    MOV       R1,A            ;off set
         MOV       A,#RD16         ;points to ls byte
         CLR       C               ;setup for subb                      
         SUBB      A,R1            ;point to proper byte
         MOV       R1,A            ;R1 is now byte pointer
LPF:     DJNZ      R7,NODP         ;is it time for dp?                  
         MOV       A,#'.'          ;put out the dp                      
         ACALL     HPPRT           ;output the dp
NODP:    MOV       A,@R1           ;get byte to output
         JB        B.0,ODD         ;even ct or odd ct                   
         SWAP      A               ;must swap if even char              
ODD:     ANL       A,#0FH          ;low nybble                          
         ADD       A,#30H          ;make number- no error chk
         ACALL     HPPRT           ;output it
         JNB       B.0,NDR0        ;if odd then dec R1 for byte
         INC       R1              ;dec byte for next
NDR0:    DJNZ      B,LPF           ;loop til done                       
         RET                       ;done                                
; done with printf routine
; *************** end of adcm 48 code ****************
        END
